<?php
// FREE VARIABLES TAB COMPONENT
//
// Copyright (c) 2010-2015 Nagios Enterprises, LLC.  All rights reserved.
//  
// $Id: freevariabletab.inc.php 38 2010-06-07 22:28:29Z swilkerson $

require_once(dirname(__FILE__).'/../componenthelper.inc.php');

// respect the name
$freevariabletab_component_name="freevariabletab";

// run the initialization function
freevariabletab_component_init();

////////////////////////////////////////////////////////////////////////
// COMPONENT INIT FUNCTIONS
////////////////////////////////////////////////////////////////////////

function freevariabletab_component_init(){
	global $freevariabletab_component_name;
	
	$versionok=freevariabletab_component_checkversion();
	
	$desc="";
	if(!$versionok)
		$desc="<b>"._("Error: This component requires Nagios XI 2009R1.2C or later.")."</b>";
	
	$args=array(

		// need a name
		COMPONENT_NAME => $freevariabletab_component_name,
		
		// informative information
		COMPONENT_AUTHOR => "Nagios Enterprises, LLC",
		COMPONENT_DESCRIPTION => _("Adds a new tabs on host and service detail screens to show free variables from config.  ").$desc,
		COMPONENT_TITLE => "Free Variable Tab",
        COMPONENT_VERSION => '1.0.1',

		// configuration function (optional)
		//COMPONENT_CONFIGFUNCTION => "freevariabletab_component_config_func",
		);
		
	register_component($freevariabletab_component_name,$args);
	
	// add a menu link
	if($versionok){
		register_callback(CALLBACK_SERVICE_TABS_INIT,'freevariabletab_component_addtab');
		register_callback(CALLBACK_HOST_TABS_INIT,'freevariabletab_component_addtab');
		}
	}
	



///////////////////////////////////////////////////////////////////////////////////////////
// MISC FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////

function freevariabletab_component_checkversion(){

	if(!function_exists('get_product_release'))
		return false;
	//requires greater than 2009R1.2
	if(get_product_release()<113)
		return false;
	// requires 2009R1.2C or higher
	if(!function_exists('get_nagios_session_protector'))
		return false;

	return true;
	}
	
function freevariabletab_component_addtab($cbtype,&$cbdata){

	// get host/service name
	$hostname=grab_array_var($cbdata,"host");
	$servicename=grab_array_var($cbdata,"service");

	$content='<div class="infotable_title">Free Variables</div>';
		
	$freevariables=freevariabletab_component_get_freevariables($hostname,$servicename);

    if($freevariables){
        $content.='<table class="table table-bordered table-striped" style="max-width: 800px;"><tr><th style="width: 200px;">Name</th><th>Value</th></tr>';
        
        foreach($freevariables->customvar as $freevariable){
            $content.='<tr><td>'.$freevariable->name.'</td><td>'.$freevariable->value.'</td></tr>';	
        }
        
        $content.='</table>';		
    
    } else
        $content.="No free variables currently set";

	$newtab=array(
		"id" => "freevariabletab",
		"title" => "Free Variables",
		"content" => $content,
         "icon" => '<i class="fa fa-tasks fa-lg"></i>',
		);

	// add new tab
	$cbdata["tabs"][]=$newtab;
	}

	
function freevariabletab_component_get_freevariables($hostname,$servicename=null){

	$args=array(
		"host_name" => $hostname
		);
    if(!$servicename){
        $x=get_xml_custom_host_variable_status($args);
        if($x->customhostvarstatus->customvars)
            return $x->customhostvarstatus->customvars;
    } else {
        $args["service_description"] = $servicename;
        $x=get_xml_custom_service_variable_status($args);
        if($x->customservicevarstatus->customvars)
            return $x->customservicevarstatus->customvars;
    }
		
	return false;
	}
	


?>