#!/bin/sh -e
# $Id: init.sh 3371 2015-05-07 22:07:36Z swilkerson $

xivar() {
	./xivar "$1" "$2"
	eval "$1"=\"\$2\"
}

# XI version
xivar xiver $(sed -n '/full/ s/.*=\(.*\)/\L\1/p' ./nagiosxi/basedir/var/xiversion)

# OS-related variables have a detailed long variable, and a more useful short
# one: distro/dist, version/ver, architecture/arch. If in doubt, use the short
. ./get-os-info
xivar distro  "$distro"
xivar version "$version"
xivar ver     "${version%%.*}" # short major version, e.g. "6" instead of "6.2"
xivar architecture "$architecture"

# Set dist variable like before (el5/el6 on both CentOS & Red Hat)
case "$distro" in
	CentOS | RedHatEnterpriseServer | oracleserver )
		xivar dist "el$ver"
		;;
	Fedora )
		xivar dist "fedora$ver"
		;;
	Debian )
		xivar dist "debian$ver"
		;;
	"SUSE LINUX" )
		xivar dist "suse$ver"
		;;
	*)
		xivar dist $(echo "$distro$ver" | tr A-Z a-z)
esac

# i386 is a more useful value than i686 for el5, because repo paths and
# package names use i386
if [ "$dist $architecture" = "el5 i686" ]; then
	xivar arch i386
else
	xivar arch "$architecture"
fi

case "$dist" in
	el5 | el6 | el7 | fedora14 )
		if [ "$arch" = "x86_64" ]; then
			xivar php_extension_dir /usr/lib64/php/modules
		fi
		;;
	suse11 | suse12 )
		if [ "$arch" = "x86_64" ]; then
			xivar php_extension_dir /usr/lib64/php5/extensions
            xivar apacheuser wwwrun
            xivar apachegroup www
            xivar httpdconfdir /etc/apache2/conf.d
            xivar httpdconf /etc/apache2/httpd.conf
            xivar httpdroot /srv/www/htdocs
            xivar phpini /etc/php5/cli/php.ini
            xivar phpconfd /etc/php5/conf.d
            xivar htpasswdbin /usr/bin/htpasswd2
            xivar httpd apache2
            xivar ntpd ntp
            xivar crond cron
            xivar mysqld mysql
		fi
		;;
    *)
		:
esac

# Set up mariadb instead of mysql for el7
if [ "$dist" == "el7" ]; then
	xivar mysqld mariadb
fi

# load xi config if present
if [ -f /usr/local/nagiosxi/html/config.inc.php ]; then
	/usr/bin/php nagiosxi/basedir/scripts/import_xiconfig.php >> xi-sys.cfg
fi
