<?php
//
// New REST API utils and classes for Nagios XI 5
//

class API extends NagiosAPI
{
    protected $user;

    public function __construct($request, $origin)
    {
        parent::__construct($request);

        $apikey = new APIKey();
        
        if (!array_key_exists('apikey', $this->request)) {
            throw new Exception('No API Key provided');
        } else if (!$apikey->verify_key($this->request['apikey'], $origin)) {
            throw new Exception('Invalid API Key');
        }
    }

    protected function objects($verb, $args)
    {
        switch ($this->method)
        {
            case 'GET':
                switch ($verb)
                {
                    case 'hoststatus':
                        $data = get_host_status_xml_output($this->request);
                        return xml2json::transformXmlStringToJson($data);
                        break;                    

                    case 'servicestatus':
                        $data = get_service_status_xml_output($this->request);
                        return xml2json::transformXmlStringToJson($data);
                        break;

                    case 'logentries':
                        $data = get_logentries_xml_output($this->request);
                        return xml2json::transformXmlStringToJson($data);
                        break;

                    case 'statehistory':
                        $data = get_statehistory_xml_output($this->request);
                        return xml2json::transformXmlStringToJson($data);
                        break;

                    case 'comment':
                        $data = get_comments_xml_output($this->request);
                        return xml2json::transformXmlStringToJson($data);
                        break;

                    case 'downtime':
                        $data = get_scheduled_downtime_xml_output();
                        return xml2json::transformXmlStringToJson($data);
                        break;

                    case 'contact':
                        $data = get_contact_objects_xml_output($this->request);
                        return xml2json::transformXmlStringToJson($data);
                        break;

                    case 'host':
                        $data = get_host_objects_xml_output($this->request);
                        return xml2json::transformXmlStringToJson($data);
                        break;

                    case 'service':
                        $data = get_service_objects_xml_output($this->request);
                        return xml2json::transformXmlStringToJson($data);
                        break;

                    case 'hostgroup':
                        $data = get_hostgroup_objects_xml_output($this->request);
                        return xml2json::transformXmlStringToJson($data);
                        break;

                    case 'servicegroup':
                        $data = get_servicegroup_objects_xml_output($this->request);
                        return xml2json::transformXmlStringToJson($data);
                        break;

                    case 'contactgroup':
                        $data = get_contactgroup_objects_xml_output($this->request);
                        return xml2json::transformXmlStringToJson($data);
                        break;

                    case 'hostgroupmembers':
                        $data = get_hostgroup_member_objects_xml_output($this->request);
                        return xml2json::transformXmlStringToJson($data);
                        break;

                    case 'servicegroupmembers':
                        $data = get_servicegroup_member_objects_xml_output($this->request);
                        return xml2json::transformXmlStringToJson($data);
                        break;

                    case 'contactgroupmembers':
                        $data = get_contactgroup_member_objects_xml_output($this->request);
                        return xml2json::transformXmlStringToJson($data);
                        break;

                    default:
                        return array('error' => _('Unknown API endpoint.'));
                        break;
                }
                break;

            case 'POST':
                return array('error' => _('Unknown API endpoint.'));
                break;

            case 'PUT':
                return array('error' => _('Unknown API endpoint.'));
                break;

            case 'DELETE':
                return array('error' => _('Unknown API endpoint.'));
                break;
        }
    }

    protected function config($verb, $args)
    {
        if (is_admin() == false) { return array('error' => _('Authenticiation failed.')); }

        switch ($this->method)
        {
            case 'GET':
                return array('info' => _('Not implemented yet. Use a POST request to add objects or a DELETE request to remove them.'));
                break;

            case 'POST':
                switch ($verb)
                {
                    case 'host':
                        return $this->create_cfg_host();
                        break;

                    case 'service':
                        return $this->create_cfg_service();
                        break;

                    case 'hostgroup':
                        return $this->create_cfg_hostgroup();
                        break;

                    case 'servicegroup':
                        return $this->create_cfg_servicegroup();
                        break;

                    //case 'command':
                    //    return $this->create_cfg_command();
                    //    break;

                    default:
                        return array('error' => _('Unknown API endpoint.'));
                        break;
                }
                break;

            case 'PUT':
                return array('info' => _('This section has not yet been implemented.'));
                break;

            case 'DELETE':
                switch ($verb)
                {
                    case 'host':
                        return $this->remove_cfg_host();
                        break;

                    case 'service':
                        return $this->remove_cfg_service();
                        break;
                }
                print_r($this->request);
                break;
        }

    }

    protected function system($verb, $args)
    {
        if (is_admin() == false) { return array('error' => _('Authenticiation failed.')); }

        switch ($this->method)
        {
            case 'GET':
                switch ($verb)
                {
                    case 'user':
                        return $this->get_users($args[0]);
                        break;

                    case 'status':
                        return $this->get_system_status($args[0]);
                        break;

                    case 'applyconfig':
                        return $this->apply_config();
                        break;

                    case 'importconfig':
                        return $this->import_config();
                        break;

                    default:
                        return array('error' => _('Unknown API endpoint.'));
                        break;
                }
                break;

            case 'POST':
                switch ($verb)
                {
                    case 'user':
                        return $this->add_user();
                        break;

                    default:
                        return array('error' => _('Unknown API endpoint.'));
                        break;
                }
                break;

            case 'PUT':
                return array('info' => _('This section has not yet been implemented.'));
                break;

            case 'DELETE':
                switch ($verb)
                {
                    case 'user':
                        return $this->delete_user($args[0]);
                        break;

                    default:
                        return array('error' => _('Unknown API endpoint.'));
                        break;
                }
                break;
        }
    }

    protected function apply_config()
    {
        submit_command(COMMAND_NAGIOSCORE_APPLYCONFIG);
        return array('success' => _('Apply config command has been sent to the backend.'));
    }

    protected function import_config()
    {
        submit_command(COMMAND_NAGIOSCORE_IMPORTONLY);
        return array('success' => _('Import configs command has been sent to the backend.'));
    }

    protected function get_users($user_id = 0)
    {
        global $sqlquery;
        global $db_tables;

        $arr = array();

        $fieldmap = array(
            "user_id" => $db_tables[DB_NAGIOSXI]["users"] . ".user_id",
            "username" => $db_tables[DB_NAGIOSXI]["users"] . ".username",
            "name" => $db_tables[DB_NAGIOSXI]["users"] . ".name",
            "email" => $db_tables[DB_NAGIOSXI]["users"] . ".email",
            "enabled" => $db_tables[DB_NAGIOSXI]["users"] . ".enabled",
        );
        $args = array(
            "sql" => $sqlquery['GetUsers'],
            "fieldmap" => $fieldmap
        );

        if (!empty($user_id)) {
            $args['useropts'] = array("user_id" => $user_id);
        }

        $sql = generate_sql_query(DB_NAGIOSXI, $args);

        if (!($rs = exec_sql_query(DB_NAGIOSXI, $sql))) {
            handle_backend_db_error(DB_NAGIOSXI);
        } else {
            $arr['records'] = $rs->RecordCount();
            if (!empty($arr['records'])) {
                $arr['users'] = array();
                while (!$rs->EOF) {
                    $arr['users'][] = array('user_id' => $rs->fields['user_id'],
                                            'username' => $rs->fields['username'],
                                            'name' => $rs->fields['name'],
                                            'email' => $rs->fields['email'],
                                            'enabled' => $rs->fields['enabled']);
                    $rs->MoveNext();
                }
            }
        }

        return $arr;
    }

    protected function add_user()
    {



    }

    protected function delete_user($user_id = 0)
    {
        if (!is_valid_user_id($user_id)) {
            return _('User with') . ' ID ' . $user_id . ' ' . _('does not exist.');
        }

        if ($user_id == $_SESSION["user_id"]) {
            return _('Cannot delete the user you are currently using.');
        }

        // Remove user from http
        update_nagioscore_cgi_config();
        $args = array(
            "username" => get_user_attr($user_id, 'username'),
        );
        submit_command(COMMAND_NAGIOSXI_DEL_HTACCESS, serialize($args));
        delete_user_id($user_id);

        return _('Success. User removed.');
    }

    protected function get_system_status()
    {
        return get_program_status_xml_output(array(), true);
    }

    // =========================
    // Configuration
    // =========================

    protected function create_cfg_host()
    {
        global $firsthost;
        $args = $this->request;
        $applyconfig = false;

        // Verify required arguments were sent
        $required = array('host_name', 'address', 'max_check_attempts', 'check_period', 'notification_interval', 'notification_period');
        $missing = array();
        foreach ($required as $r) {
            if (!array_key_exists($r, $args)) {
                $missing[] = $r;
            }
        }

        if (!array_key_exists('contacts', $args) && !array_key_exists('contact_groups', $args)) {
            $missing[] = 'contacts OR contact_groups';
        }

        if (count($missing) > 0) {
            return array('error' => _('Missing required variables'), 'missing' => $missing);
        }

        // Check if we should applyconfig nagios after
        if (array_key_exists('applyconfig', $args)) {
            if ($args['applyconfig'] == 1) {
                $applyconfig = true;
            }
        }

        $obj = array("type" => OBJECTTYPE_HOST);
        foreach ($required as $r) {
            $obj[$r] = $args[$r];
        }

        // Add any optional components that we need to
        $others = array('alias', 'display_name', 'parents', 'hourly_value', 'hostgroups', 'check_command', 'use', 'initial_state', 'check_interval', 'retry_interval', 'active_checks_enabled', 'passive_checks_enabled', 'obsess_over_host', 'check_freshness', 'freshness_threshold', 'event_handler', 'event_handler_enabled', 'low_flap_threshold', 'high_flap_threshold', 'flap_detection_enabled', 'flap_detection_options', 'process_perf_data', 'retain_status_information', 'retain_nonstatus_information', 'first_notification_delay', 'notification_options', 'notifications_enabled', 'stalking_options', 'notes', 'notes_url', 'action_url', 'icon_image', 'icon_image_alt', 'vrml_image', 'statusmap_image', '2d_coords', '3d_coords');

        foreach ($others as $o) {
            if (array_key_exists($o, $args)) {
                $obj[$o] = $args[$o];
            }
        }

        // Commit the data to a file
        $objs = array($obj);
        commit_cfg_objects($objs, $firsthost);

        if ($applyconfig) {
            submit_command(COMMAND_NAGIOSCORE_APPLYCONFIG);
            $msg = ' ' . _('Config applied, Nagios Core was restarted.');
        } else {
            submit_command(COMMAND_NAGIOSCORE_IMPORTONLY);
            $msg = ' ' . _('Config imported but not yet applied.');
        }

        return array('success' => _('Successfully added') . ' ' . $args['host_name'] . ' ' . _('to the system.') . $msg);
    }

    protected function create_cfg_service()
    {
        global $firsthost;
        $args = $this->request;
        $applyconfig = false;

        // Verify required arguments were sent
        $required = array('host_name', 'service_description', 'check_command', 'max_check_attempts', 'check_interval', 'retry_interval', 'check_period', 'notification_interval', 'notification_period');
        $missing = array();
        foreach ($required as $r) {
            if (!array_key_exists($r, $args)) {
                $missing[] = $r;
            }
        }

        if (!array_key_exists('contacts', $args) && !array_key_exists('contact_groups', $args)) {
            $missing[] = 'contacts OR contact_groups';
        }

        if (count($missing) > 0) {
            return array('error' => _('Missing required variables'), 'missing' => $missing);
        }

        // Check if we should applyconfig nagios after
        if (array_key_exists('applyconfig', $args)) {
            if ($args['applyconfig'] == 1) {
                $applyconfig = true;
            }
        }

        $obj = array("type" => OBJECTTYPE_SERVICE);
        foreach ($required as $r) {
            $obj[$r] = $args[$r];
        }

        // Add any optional components that we need to
        $others = array('hostgroup_name', 'display_name', 'parents', 'hourly_value', 'servicegroups', 'is_volatile', 'use', 'initial_state', 'active_checks_enabled', 'passive_checks_enabled', 'obsess_over_service', 'check_freshness', 'freshness_threshold', 'event_handler', 'event_handler_enabled', 'low_flap_threshold', 'high_flap_threshold', 'flap_detection_enabled', 'flap_detection_options', 'process_perf_data', 'retain_status_information', 'retain_nonstatus_information', 'first_notification_delay', 'notification_options', 'notifications_enabled', 'stalking_options', 'notes', 'notes_url', 'action_url', 'icon_image', 'icon_image_alt');

        foreach ($others as $o) {
            if (array_key_exists($o, $args)) {
                $obj[$o] = $args[$o];
            }
        }

        // Commit the data to a file
        $objs = array($obj);
        commit_cfg_objects($objs, $firsthost);

        if ($applyconfig) {
            submit_command(COMMAND_NAGIOSCORE_APPLYCONFIG);
            $msg = ' ' . _('Config applied, Nagios Core was restarted.');
        } else {
            submit_command(COMMAND_NAGIOSCORE_IMPORTONLY);
            $msg = ' ' . _('Config imported but not yet applied.');
        }

        return array('success' => _('Successfully added') . ' ' . $args['host_name'] . ' :: ' . $args['service_description'] . ' ' . _('to the system.') . $msg);
    }

    protected function create_cfg_hostgroup()
    {
        global $firsthost;
        $args = $this->request;
        $applyconfig = false;

        $required = array('hostgroup_name', 'alias');
        $missing = array();
        foreach ($required as $r) {
            if (!array_key_exists($r, $args)) {
                $missing[] = $r;
            }
        }

        if (count($missing) > 0) {
            return array('error' => _('Missing required variables'), 'missing' => $missing);
        }

        // Check if we should applyconfig nagios after
        if (array_key_exists('applyconfig', $args)) {
            if ($args['applyconfig'] == 1) {
                $applyconfig = true;
            }
        }

        $obj = array("type" => OBJECTTYPE_HOSTGROUP);
        foreach ($required as $r) {
            $obj[$r] = $args[$r];
        }

        $other = array('members', 'hostgroup_members', 'notes', 'notes_url', 'action_url');

        foreach ($others as $o) {
            if (array_key_exists($o, $args)) {
                $obj[$o] = $args[$o];
            }
        }

        // Commit the data to a file
        $objs = array($obj);
        commit_cfg_objects($objs, $firsthost);

        if ($applyconfig) {
            submit_command(COMMAND_NAGIOSCORE_APPLYCONFIG);
            $msg = ' ' . _('Config applied, Nagios Core was restarted.');
        } else {
            submit_command(COMMAND_NAGIOSCORE_IMPORTONLY);
            $msg = ' ' . _('Config imported but not yet applied.');
        }

        return array('success' => _('Successfully added') . ' ' . $args['hostgroup_name'] . ' ' . _('to the system.') . $msg);
    }

    protected function create_cfg_servicegroup()
    {
        global $firsthost;
        $args = $this->request;
        $applyconfig = false;

        $required = array('servicegroup_name', 'alias');
        $missing = array();
        foreach ($required as $r) {
            if (!array_key_exists($r, $args)) {
                $missing[] = $r;
            }
        }

        if (count($missing) > 0) {
            return array('error' => _('Missing required variables'), 'missing' => $missing);
        }

        // Check if we should applyconfig nagios after
        if (array_key_exists('applyconfig', $args)) {
            if ($args['applyconfig'] == 1) {
                $applyconfig = true;
            }
        }

        $obj = array("type" => OBJECTTYPE_SERVICEGROUP);
        foreach ($required as $r) {
            $obj[$r] = $args[$r];
        }

        $other = array('members', 'servicegroup_members', 'notes', 'notes_url', 'action_url');

        foreach ($others as $o) {
            if (array_key_exists($o, $args)) {
                $obj[$o] = $args[$o];
            }
        }

        // Commit the data to a file
        $objs = array($obj);
        commit_cfg_objects($objs, $firsthost);

        if ($applyconfig) {
            submit_command(COMMAND_NAGIOSCORE_APPLYCONFIG);
            $msg = ' ' . _('Config applied, Nagios Core was restarted.');
        } else {
            submit_command(COMMAND_NAGIOSCORE_IMPORTONLY);
            $msg = ' ' . _('Config imported but not yet applied.');
        }

        return array('success' => _('Successfully added') . ' ' . $args['servicegroup_name'] . ' ' . _('to the system.') . $msg);
    }

    protected function create_cfg_command()
    {
        $args = $this->request;
        $applyconfig = true;
        

    }

    protected function remove_cfg_host()
    {
        $applyconfig = false;
        $missing = array();
        
        if (!array_key_exists('host_name', $this->request)) { $missing[] = 'host_name'; }

        if (count($missing) > 0) {
            return array('error' => _('Missing required variables'), 'missing' => $missing);
        }

        $host = grab_array_var($this->request, 'host_name', '');
        $hid = nagiosql_get_host_id($host);

        if ($hid <= 0) {
            return array('error' => _('Could not find a unique id for this host.'));
        }

        if (can_host_be_deleted($host) == false) {
            return array('error' => _('Host cannot be deleted using this method. Must be deleted through the CCM.'));
        }

        // Check if we should applyconfig nagios after
        if (array_key_exists('applyconfig', $this->request)) {
            if ($this->request['applyconfig'] == 1) {
                $applyconfig = true;
            }
        }

        if ($applyconfig) {
            $msg = ' ' . _('Config applied, Nagios Core was restarted.');
        } else {
            $msg = ' ' . _('Config imported but not yet applied.');
        }

        delete_nagioscore_host($host, $applyconfig);
        return array('success' => _('Removed') . ' ' . $host . ' ' . _('from the system.') . $msg);
    }

    protected function remove_cfg_service()
    {
        $applyconfig = false;
        $missing = array();

        if (!array_key_exists('host_name', $this->request)) { $missing[] = 'host_name'; }
        if (!array_key_exists('service_description', $this->request)) { $missing[] = 'service_description'; }
        
        if (count($missing) > 0) {
            return array('error' => _('Missing required variables'), 'missing' => $missing);
        }

        $host = grab_array_var($this->request, 'host_name', '');
        $service = grab_array_var($this->request, 'service_description', '');
        $sid = nagiosql_get_service_id($host, $service);

        if ($sid <= 0) {
            return array('error' => _('Could not find a unique id for this service.'));
        }

        if (can_service_be_deleted($host, $service) == false) {
            return array('error' => _('Service cannot be deleted using this method. Must be deleted through the CCM.'));
        }

        // Check if we should applyconfig nagios after
        if (array_key_exists('applyconfig', $this->request)) {
            if ($this->request['applyconfig'] == 1) {
                $applyconfig = true;
            }
        }

        if ($applyconfig) {
            $msg = ' ' . _('Config applied, Nagios Core was restarted.');
        } else {
            $msg = ' ' . _('Config imported but not yet applied.');
        }

        delete_nagioscore_service($host, $service, $applyconfig);
        return array('success' => _('Removed') . ' ' . $host . ' :: ' . $service . ' ' . _('from the system.') . $msg);
    }

}


/**
 * @param $objects
 * @param $firsthost
 */
function commit_cfg_objects($objects, &$firsthost)
{
    $have_first_host = false;

    // FIRST PROCESS NON-SERVICES

    // Create the import file
    $fname = "";
    $fh = create_nagioscore_import_file($fname);

    foreach ($objects as $oid => $obj) {

        // Get the object type
        $oname = "";
        switch ($obj["type"]) {
            case OBJECTTYPE_HOST:
                $oname = "host";
                break;
            case OBJECTTYPE_HOSTGROUP:
                $oname = "hostgroup";
                break;
            case OBJECTTYPE_SERVICEGROUP:
                $oname = "servicegroup";
                break;
            case OBJECTTYPE_COMMAND:
                $oname = "command";
                break;
            default:
                break;
        }

        // Unhandled object types
        if ($oname == "") {
            continue;
        }

        // Write the object definition to file
        fprintf($fh, "define " . $oname . "{\n");

        foreach ($obj as $var => $val) {
            if ($var == "type")
                continue;
            fprintf($fh, $var . "\t" . $val . "\n");

            if ($oname == "host" && $have_first_host == false && $var == "host_name") {
                $have_first_host = true;
                $firsthost = $val;

                // Preload ndoutils with host
                add_ndoutils_object(OBJECTTYPE_HOST, $firsthost);
            }
        }
        fprintf($fh, "}\n");
    }

    // Commit the import file
    fclose($fh);
    commit_nagioscore_import_file($fname);

    // PROCESS SERVICES
    $openfile = false;
    $importfiles = array();
    $multihost = false;
    foreach ($objects as $oid => $obj) {

        if ($have_first_host == false) {
            $have_first_host = true;
            $firsthost = $obj["host_name"];
        } else {
            if ($firsthost != $obj["host_name"])
                $multihost = true;
        }

        // Get the object type
        $oname = "";
        switch ($obj["type"]) {
            case OBJECTTYPE_SERVICE:
                $oname = "service";

                // Preload ndoutils with service
                add_ndoutils_object(OBJECTTYPE_SERVICE, $obj["host_name"], $obj["service_description"]);

                break;
            default:
                break;
        }

        // unhandled object types
        if ($oname == "")
            continue;

        // create the import file
        if ($openfile == false) {
            $fname = $obj["host_name"];
            $fh = create_nagioscore_import_file($fname);
            $openfile = true;
        }

        // write the object definition to file
        fprintf($fh, "define " . $oname . "{\n");

        foreach ($obj as $var => $val) {
            if ($var == "type")
                continue;
            fputs($fh, $var . "\t" . $val . "\n");
        }
        fprintf($fh, "}\n");

        // close file now, commit the import file later
        if ($openfile == true) {
            $openfile = false;
            fclose($fh);
            // add this to the import files to commit (if it isn't already in there)
            if (!in_array($fname, $importfiles))
                $importfiles[] = $fname;
        }
    }

    // commit all import files
    foreach ($importfiles as $if) {
        commit_nagioscore_import_file($if);
    }

    // multiple hosts
    if ($multihost == true)
        $firsthost = "";
}


function get_scheduled_downtime_xml_output()
{
    global $sqlquery;
    global $db_tables;
    global $request;

    // generate query
    $fieldmap = array(
        "instance_id" => $db_tables[DB_NDOUTILS]["instances"] . ".instance_id",
        "downtime_type" => $db_tables[DB_NDOUTILS]["scheduleddowntime"] . ".downtime_type",
        "entry_time" => $db_tables[DB_NDOUTILS]["scheduleddowntime"] . ".entry_time",
        "author_name" => $db_tables[DB_NDOUTILS]["scheduleddowntime"] . ".author_name",
        "comment_data" => $db_tables[DB_NDOUTILS]["scheduleddowntime"] . ".comment_data",
        "internal_id" => $db_tables[DB_NDOUTILS]["scheduleddowntime"] . ".internal_downtime_id",
        "triggered_by" => $db_tables[DB_NDOUTILS]["scheduleddowntime"] . ".triggered_by_id",
        "fixed" => $db_tables[DB_NDOUTILS]["scheduleddowntime"] . ".is_fixed",
        "duration" => $db_tables[DB_NDOUTILS]["scheduleddowntime"] . ".duration",
        "scheduled_start_time" => $db_tables[DB_NDOUTILS]["scheduleddowntime"] . ".scheduled_start_time",
        "scheduled_end_time" => $db_tables[DB_NDOUTILS]["scheduleddowntime"] . ".scheduled_end_time",
        "was_started" => $db_tables[DB_NDOUTILS]["scheduleddowntime"] . ".was_started",
        "actual_start_time" => $db_tables[DB_NDOUTILS]["scheduleddowntime"] . ".actual_start_time",
        "actual_start_time_usec" => $db_tables[DB_NDOUTILS]["scheduleddowntime"] . ".actual_start_time_usec",
        "object_id" => $db_tables[DB_NDOUTILS]["scheduleddowntime"] . ".object_id",
        "objecttype_id" => "obj1.objecttype_id",
        "host_name" => "obj1.name1",
        "service_description" => "obj1.name2"
    );
    $objectauthfields = array(
        "object_id"
    );
    $instanceauthfields = array(
        "instance_id"
    );
    $default_order = "scheduled_start_time DESC, scheduleddowntime_id DESC";
    $args = array(
        "sql" => $sqlquery['GetScheduledDowntime'],
        "fieldmap" => $fieldmap,
        "objectauthfields" => $objectauthfields,
        "objectauthperms" => P_READ,
        "instanceauthfields" => $instanceauthfields,
        "default_order" => $default_order
    );
    $sql = generate_sql_query(DB_NDOUTILS, $args);

    if (!($rs = exec_sql_query(DB_NDOUTILS, $sql))) {
        handle_backend_db_error(DB_NDOUTILS);
    } else {

        // Generate the XML
        $outputtype = grab_request_var("outputtype", "");
        $output = "<scheduleddowntimelist>\n";
        $output .= "  <recordcount>" . $rs->RecordCount() . "</recordcount>\n";

        if (!isset($request["totals"])) {
            while (!$rs->EOF) {
                if ($outputtype == "json") {

                } else {
                    $output .= "  <scheduleddowntime id='" . db_field($rs, 'scheduleddowntime_id') . "'>\n";
                }
                $output .= xml_db_field(2, $rs, 'instance_id', '', true);
                $output .= xml_db_field(2, $rs, 'downtime_type', '', true);
                $output .= xml_db_field(2, $rs, 'object_id', '', true);
                $output .= xml_db_field(2, $rs, 'objecttype_id', '', true);
                $output .= xml_db_field(2, $rs, 'host_name', '', true);
                $output .= xml_db_field(2, $rs, 'service_description', '', true);
                $output .= xml_db_field(2, $rs, 'entry_time', '', true);
                $output .= xml_db_field(2, $rs, 'author_name', '', true);
                $output .= xml_db_field(2, $rs, 'comment_data', '', true);
                $output .= xml_db_field(2, $rs, 'internal_downtime_id', 'internal_id', true);
                $output .= xml_db_field(2, $rs, 'triggered_by_id', 'triggered_by', true);
                $output .= xml_db_field(2, $rs, 'is_fixed', 'fixed', true);
                $output .= xml_db_field(2, $rs, 'duration', '', true);
                $output .= xml_db_field(2, $rs, 'scheduled_start_time', '', true);
                $output .= xml_db_field(2, $rs, 'scheduled_end_time', '', true);
                $output .= xml_db_field(2, $rs, 'was_started', '', true);
                $output .= xml_db_field(2, $rs, 'actual_start_time', '', true);
                $output .= xml_db_field(2, $rs, 'actual_start_time_usec', '', true);
                $output .= "  </scheduleddowntime>\n";
                $rs->MoveNext();
            }
        }

        $output .= "</scheduleddowntimelist>\n";

        return $output;
    }
}


function db_field($rs, $fieldname)
{
    return get_xml_db_field_val($rs, $fieldname);
}


function xml_db_field($level, $rs, $fieldname, $nodename = "", $return = false)
{
    $temp = get_xml_db_field($level, $rs, $fieldname, $nodename);
    if ($return) {
        return $temp;
    } else {
        echo $temp;
    }
}