<?php
//
//  Nagios Core Config Manager
//  Copyright (c) 2010-2015 Nagios Enterprises, LLC
//
//  File: common_settings.php
//  Desc: Creates the HTML for the "Common Settings" tab in object management pages. Used in the
//        form class to output the area where everything is defined.
//
?>
<div id="visibleDivs">
    <div id="tab1">
        <h3><?php echo _("Common Settings"); ?></h3>
        <div class="leftBox">
            <?php
            // Host or Host Template only items
            if ($this->exactType == 'host' || $this->exactType =='hosttemplate') { 
                $tfName = (($this->exactType == 'host') ? _('Host Name') : _('Template Name')); 
                $name_field = (($this->exactType == 'host') ? 'host_name' : 'template_name');
            ?>
            <div class="ccm-row">
                <label for="tfName"><?php echo $tfName; ?> <span class="req">*</span></label>
                <input name="tfName" class="tf_wide required" type="text" id="tfName" value="<?php val($FIELDS[$name_field]); ?>">
            </div>
            <div class="ccm-row">
                <label for="tfFriendly"><?php echo _("Description"); ?></label>
                <input name="tfFriendly" class="tf_wide" type="text" id="tfFriendly" value="<?php val(encode_form_val($FIELDS['alias'])); ?>">
            </div>
            <?php
            // Host only items 
            if ($this->exactType == 'host') {
            ?>
            <div class="ccm-row">
                <label for="tfAddress"><?php echo _("Address"); ?> <span class="req">*</span></label>
                <input name="tfAddress" type="text" class="tf_wide required" id="tfAddress" value="<?php val(encode_form_val($FIELDS['address'])); ?>">
            </div>
            <div class="ccm-row">
                <label for="tfDisplayName">Display name</label>
                <input name="tfDisplayName" type="text" class="tf_wide" id="tfDisplayName" value="<?php val(encode_form_val($FIELDS['display_name'])); ?>">
            </div>
            <?php } ?>
            <p><a class="wideLinkBox" href="javascript:overlay('parentBox')" title="Manage Parents"><?php echo _("Manage Parents"); ?></a></p>
            <?php
            }
    
            // Service or Service Template only
            if ($this->exactType == 'service' || $this->exactType == 'servicetemplate' ) {
                $tfName = (($this->exactType == 'service') ? _('Config Name') : _('Template Name'));
                $required = (($this->exactType == 'service') ? 'required' : '');
            ?>
            <div class="ccm-row">
                <label for="tfName"><?php echo $tfName; ?> <span class="req">*</span></label>
                <input name="tfName" class="tf_wide required" type="text" id="tfName" value="<?php echo val($FIELDS['config_name'], false).val($FIELDS['template_name'], false); ?>">
            </div>
            <div class="ccm-row">
                <label for="tfFriendly"><?php echo _('Description'); ?> <span class="req">*</span></label>
                <input name="tfServiceDescription" class="tf_wide <?php echo $required; ?>" type="text" id="tfServiceDescription" value="<?php val($FIELDS['service_description']); ?>">
            </div>
            <div class="ccm-row">
                <label for="tfDisplayName"><?php echo _('Display name'); ?></label>
                <input name="tfDisplayName" type="text" class="tf_wide" id="tfDisplayName" value="<?php val(encode_form_val($FIELDS['display_name'])); ?>">
            </div>
            <p><a class="wideLinkBox" href="javascript:overlay('hostBox')" title="Assign to Hosts"><?php echo _("Manage Hosts"); ?></a></p>
            <?php
            }
            ?>
            <p><a class='wideLinkBox' href="javascript:overlay('templateBox')" title="Manage Templates"><?php echo _("Manage Templates"); ?></a></p>
            <p><a class='wideLinkBox' href="javascript:overlay('hostgroupBox')" title="Manage Hostgroups"><?php echo _("Manage Hostgroups"); ?></a></p>
            <?php
            // Serivce or Service Template only
            if ($this->exactType == 'service' || $this->exactType == 'servicetemplate') {
            ?>
            <p><a class='wideLinkBox' href="javascript:overlay('servicegroupBox')" title="Manage Servicegroups"><?php echo _("Manage Servicegroups"); ?></a></p>
            <?php
            }

            // Check if the active button should be checked
            $active_checked = '';
            if ((isset($FIELDS['active']) && $FIELDS['active'] == '1') || !isset($FIELDS['active'])) {
                $active_checked = 'checked="checked"';
            }
            ?>
            <div class="ccm-row oneline">
                <label for="chbActive"><?php echo _("Active"); ?></label>
                <input name="chbActive" type="checkbox" class="checkbox" id="chbActive" value="1" <?php echo $active_checked; ?>>
                <i class="fa fa-info-circle tooltip-info" title="<?php echo _("Only active objects will be written to the config files and appear in Nagios. Inactive objects will only be shown in the CCM."); ?>"></i>
            </div>
        </div>
        <div class="rightBox">
            <div class="ccm-row">
                <label for="selHostCommand"><?php echo _('Check command'); ?></label>
                <select name="selHostCommand" id="selHostCommand" onchange="reveal_command(this.value);">
                    <?php
                    // Host commands
                    $selected = "";
                    if ($FIELDS['sel_check_command'] == '0') { $selected = ' selected="selected"'; }
                    echo '<option value="0"'.$selected.'>&nbsp;</option>';
                    
                    foreach ($FIELDS['selCommandOpts'] as $opt) {
                        $selected = "";
                        if ($opt['active'] != 1) {
                            continue;
                        }
                        if ($FIELDS['sel_check_command'] == $opt['id']) { $selected = ' selected="selected"'; }
                        echo '<option value="'.$opt['id'].'"'.$selected.'>'.$opt['command_name'].'</option>';
                    }
                    ?>
                </select>
            </div>
            <div class="ccm-row">
                <label for="fullcommand"><?php echo _('Command view'); ?></label>
                <pre><div id="fullcommand"><?php if (!empty($FIELDS['fullcommand'])) { val($FIELDS['fullcommand']); } else { echo _("No command selected"); } ?></div></pre>
            </div>
            <div class="ccm-row oneline">
                <label for="tfArg1">$ARG1$</label>
                <input name="tfArg1" class="tf_wide arg" type="text" id="tfArg1" value="<?php val(encode_form_val($FIELDS['tfArg1'])) ?>">
            </div>
            <div class="ccm-row oneline">
                <label for="tfArg2">$ARG2$</label>
                <input name="tfArg2" class="tf_wide arg" type="text" id="tfArg2" value="<?php val(encode_form_val($FIELDS['tfArg2'])) ?>">
            </div>
            <div class="ccm-row oneline">
                <label for="tfArg3">$ARG3$</label>
                <input name="tfArg3" class="tf_wide arg" type="text" id="tfArg3" value="<?php val(encode_form_val($FIELDS['tfArg3'])) ?>">
            </div>
            <div class="ccm-row oneline">
                <label for="tfArg4">$ARG4$</label>
                <input name="tfArg4" class="tf_wide arg" type="text" id="tfArg4" value="<?php @val(encode_form_val($FIELDS['tfArg4'])) ?>">
            </div>
            <div class="ccm-row oneline">
                <label for="tfArg5">$ARG5$</label>
                <input name="tfArg5" class="tf_wide arg" type="text" id="tfArg5" value="<?php @val(encode_form_val($FIELDS['tfArg5'])) ?>">
            </div>
            <div class="ccm-row oneline">
                <label for="tfArg6">$ARG6$</label>
                <input name="tfArg6" class="tf_wide arg" type="text" id="tfArg6" value="<?php @val(encode_form_val($FIELDS['tfArg6'])) ?>">
            </div>
            <div class="ccm-row oneline">
                <label for="tfArg7">$ARG7$</label>
                <input name="tfArg7" class="tf_wide arg" type="text" id="tfArg7" value="<?php @val(encode_form_val($FIELDS['tfArg7'])) ?>">
            </div>
            <div class="ccm-row oneline">
                <label for="tfArg8">$ARG8$</label>
                <input name="tfArg8" class="tf_wide arg" type="text" id="tfArg8" value="<?php @val(encode_form_val($FIELDS['tfArg8'])) ?>">
            </div>
            <div id="command_test_box"><a class="wideLinkBox" href="javascript:void(0);" id="command_test"><?php echo _("Test Check Command"); ?></a></div>
        </div>
    </div>